#!/usr/bin/perl
#-----------------------------------------------------------------------
# dlg_image.cgi: custom dialog used by HTML editor for adding and editing 
# anchor tags
#-----------------------------------------------------------------------
#
#   Copyright 2001-2004 Exware Solutions, Inc.  http://www.exware.com
#
#   This file is part of ExSite WebWare (ExSite, for short).
#
#   ExSite is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   ExSite is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with ExSite; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Users requiring warranty coverage and/or support may arrange alternate
#   commercial licensing for ExSite, by contacting Exware Solutions 
#   via the website noted above.
#
#----------------------------------------------------------------------------

use strict;
use ExSite::Config;
use ExSite::Misc;
use ExSite::DB;
use ExSite::Content;
use ExSite::Section;
use ExSite::Page;
use ExSite::Image;
use myConfig;
use ExSite::Dialog;

&exsite_init;

$config{page_buffering} = 0;
my %data = &DecodeInput;  # accepts input by post or query
my $db = new ExSite::DB;
my $access = $db->authorize;
my $dlg = new ExSite::Dialog;
my $title = "Insert Image/Document"; # title
my $file = "dlg_image.cgi";
my $defaultimagedim = 200;
my $selectedlib = 0;
my $OUT = "";

if ($access==0){
	print "Location: login.cgi\n\n";
}elsif ($access < 2){
	$OUT .= &PrintHeader;
	$OUT .= "<p>Access denied.  Webmasters only.</p>";
	$OUT .= &PrintFooter;
	print $OUT;
}elsif ($data{action} eq "getimagelist"){
	# get image/document list
	print &getImageList;
}elsif ($data{action} eq "getimage"){
	# get single image/document
	print &getImage;
}elsif ($data{action} eq "geturl"){
	# get img url
	print &getUrl;
}else{
	#&findSectionID; # figure out section id
	if (! $data{section_id}) {
	    $data{section_id} = $dlg->find_section_id(\%data);
	}
	$OUT .= &PrintHeader;
	# process image/document upload
	$OUT .= $data{action} eq "upload_file" ? &processUpload : "";
	$OUT .= &PrintForm;
	$OUT .= &PrintFooter;
	print $OUT;
}
#--------------------------------------------------------------------------------------
# figure out section id
sub findSectionID{
	my ($cdata);
	return if ($data{section_id});
	if (exists $data{data_id}) { 
	    $cdata = new ExSite::ContentData(id=>$data{data_id});
	}else {
	    $cdata = new ExSite::Page(id=>$data{cont_id});
	}
	$cdata->set_context();
	# set section id
	$data{section_id} = $cdata->get("section","section_id");
	return;
}
# css style sheet
sub getCss{
	my $out;
	$out .= <<END;
<style>
	BODY { 
		background-color: buttonface; 
	}
	TABLE.dlg { 
		border:0; 
	}
	.dlg TD { 
		horizontal-align: left; height: 20px; 
	}
	.dlg INPUT { 
		border-size: 2px; 
	}
END
	# for msie
	if ($ENV{'HTTP_USER_AGENT'} =~ /msie/i){
		$out .= <<END;	
	TD, INPUT, SELECT { 
		font-family: "MS Sans Serif"; font-size: xx-small; 
		// vertical-align: middle; 
	}
	INPUT.button { 
		border-top: 1px solid white; border-left: 1px solid white;
		border-bottom: 1px solid black; border-right: 1px solid black;
		font-size: x-small; width: 60px; }
	SELECT { height: 75%; }
END
	}else{	
		$out .= <<END;
	TD, INPUT, P { 
		font-family: sans-serif; font-size: small; 
		// vertical-align: middle; 
	}
	INPUT.button_style { 
		border-top: 1px solid white; 
		border-left: 1px solid white;
		border-bottom: 1px solid black; 
		border-right: 1px solid black;
		font-size: x-small; 
		width: 60px;
	}
END
	}
	$out .= "\n</style>";
	return $out;
}
# javascript
sub getJS{
	return <<END;
<script langauge="JavaScript" src="$config{server}{HTMLpath}/_ExSite/js/httprequest.js"></script>
<script language="JavaScript">
<!--
	var isImage = 0;
	function insertimage(){
		var objName,objAlign,objId,objUrl,re,tempImage;
		re = new RegExp('checker');
		tempImage = new Image();
		objName = getContentName(document.forms[0].obj_id.value);
		objId = getContentID(document.forms[0].obj_id.value);
		objUrl = getRequestData('$config{server}{CGIpath}/$file?action=geturl&content_id=' + objId);
		objAlign = document.forms[0].align.value;
		tempImage.src = objUrl;
		if (re.test(objUrl)){
			tempImage.width = 100;
			tempImage.height = 50;
		}
		objWidth = tempImage.width;
		objHeight = tempImage.height;
		window.opener.getImage(isImage,objName,objUrl,objAlign,objWidth,objHeight);
		window.close();
	}
	function cancel(){
		window.returnValue = null;
		window.close();
	}
	function showimage(){
		var target,image,requestUrl,re;
		re = new RegExp('img');
		target = document.getElementById('disp');
		id = getContentID(document.forms[0].obj_id.value);
		if (!id){
			return;
		}
		requestUrl = '$config{server}{CGIpath}/$file?action=getimage&content_id=' + id;
		image = getRequestData(requestUrl);
		if (re.test(image)){
			showAlignmentForm();
		}else{
			hideAlignmentForm();
		}
		target.innerHTML = image;
	}
	function getImageList(){
		var target,lib_id,requestUrl,list;
		hideAlignmentForm();
		clearPreviewBlock();
		lib_id = document.forms[0].lib_id.value;
		target = document.getElementById('imageSelector');
		requestUrl = '$config{server}{CGIpath}/$file?action=getimagelist&page_id=' + lib_id;
		list = getRequestData(requestUrl);
		target.innerHTML = list;
	}
	function hideAlignmentForm(){
		var alignment = document.getElementById('img_alignment');
		alignment.style.visibility = 'hidden';
		isImage = 0;
	}
	function showAlignmentForm(){
		var alignment = document.getElementById('img_alignment');
		alignment.style.visibility = 'visible';
		isImage = 1;
	}
	function clearPreviewBlock(){
		var preview = document.getElementById('disp');
		preview.innerHTML = '';
	}
	function getContentID(val){
		var stringsplt = val.split(':');
		return stringsplt[0];
	}
	function getContentName(val){
		var result,stringsplt;
		result = '';
		stringsplt = val.split(':');
		for(var i = 1; i < stringsplt.length; i++){
			result += stringsplt[i];
		}
		return result;
	}
//-->
</script>
END
}
sub PrintHeader {
	my (@links) = @_;
	my $css = &getCss;
	my $js = &getJS;
	return <<END;
Content-type: text/html


<HTML>
<HEAD>
<TITLE>$title</TITLE>
$css
$js
</head>
<body topmargin="0" leftmargin="0" style="border: 0; margin: 0;" scroll="no">
END
}
sub PrintFooter {
	return "\n</body>\n</html>";
}
# build insert image/document dialog
sub PrintForm{
	# library drop down list
	my $library_selector= &getLibrarySelector;
	# image drop down list
	my $image_selector = &getImageSelector;
	# image alignment options
	my $image_attribute = &getAlignmentRadio;
	# preview area
	my $image_preview = &getPreviewBlock;
	my $out = <<END;
<form onSubmit="insertimage()">
<table border="0" cellspacing="10" width="100%">
<tr>
	<td colspan="2">
	<h3 align="center">$title</h3>
	</td>
</tr>
<tr>
	<td valign="top">
	$library_selector
	<br><br>
	$image_selector
	<br><br>
	$image_attribute
	<br><br>
	<input type="button" value="Insert" onClick="insertimage();">
	&nbsp;&nbsp;&nbsp;&nbsp;
	<input type="button" value="Cancel" onClick="cancel();">
	</td>
	<td bgcolor="white" valign="top" width="150" align="center" style="border-left:solid;border-right:solid;border-top:solid;border-bottom:solid;">
	$image_preview
	</td>
</tr>
</form>
END
	# show upload form if it's turned on
	if ($config{form}{htmleditor}{op_image_upload}){
		my $upload_form = &uploadForm();
		$out .= <<END;
<tr>
	<td colspan="2">
	<hr>
	</td>
</tr>
<tr>
	<td colspan="2">
	$upload_form
	</td>
</tr>
END
	}
	# set default library
	$out .= &setDefaultLib;
	return $out;
}
# document upload form
sub uploadForm{
	my $s = new ExSite::Section(id=>$data{section_id});
	my @libs = $s->fetch_libraries;
	my $select;
	foreach my $l (@libs){
		if ($db->owns($l->{section_id})){
			$select .= "\n<option value=\"$l->{page_id}\">$l->{filename}</option>";
		}
	}
	
	return <<END;
<form method="post" enctype="multipart/form-data" action="$config{server}{CGIpath}/$file">
<input type="hidden" name="action" value="upload_file">
<input type="hidden" name="section_id" value="$data{section_id}">
<input type="hidden" name="cont_id" value="$data{cont_id}">
<table border="0" cellspacing="0" width="100%">
<tr>
	<td colspan="3">
	<b>Add an image/document to a library:</b>
	</td>
</tr>
<tr>
	<td>
	Library&nbsp;
	<select name="library">
	<option value="">== Select ==</option>
	$select		
	</select>
	</td>
	<td>
	File&nbsp;<input type="file" name="fileupload">
	</td>
	<td>
	<input type="submit" value="Upload">
	</td>
</tr>
</table>
</form>
END
}
# javascript popup
sub alertPopup{
	my ($msg) = @_;
	return <<END;
<script language="javascript">
<!--
alert('$msg');
// -->
</script>
END
}
# process form data
sub processUpload{
	my $space = "&nbsp;&nbsp;";
	return &alertPopup("Error: File upload failed.  No library is selected.") if !$data{library};
	return &alertPopup("Error: File upload failed.  No file is selected.") if !$data{fileupload};
	my $filename;
	my $rawdata = undef;
	my $size = 0;
	my $mimedata = "";
	my $file = $data{fileupload};
	my $continue = 1;
	# read upload data
	while ($continue){
		$continue = read($file,$rawdata,1024,$size);
		$size += $continue;
	}
	return &alertPopup("Error: File upload failed.  No file data was received.") if !$size;
	my @path = split /[:\\\/]/, $file;        
        if ($path[-1]){
		$filename = &url_escape($path[-1]);
	}
	my $img = new ExSite::Image($filename,$rawdata);
	my $fdata = $img->encode;
	my $content = new ExSite::Content;
	# insert form data
	my $cid = $content->make_content({name=>$filename,page_id=>$data{library}},{fdata=>$fdata});
	if ($cid){
		return &alertPopup("$filename has been uploaded successfully.");
	}else{
		my @errors = $content->fetch_diagnostics("error");
		my $error_message = join("\\n",@errors);
		return &alertPopup($error_message);
	}
	return;
}
# create an image/document list
sub getImageList{
	my ($out,$list);
	$list = $db->fetch_match("content",{page_id=>$data{page_id}});
	my $selector;
	foreach my $l (@$list){
		$selector .= "\n<option value=\"$l->{content_id}:$l->{name}\">$l->{name}</option>";
	}
	return <<END;
Expires: -1
Cache-Control: no-cache
Cache-Control: max-page=0
Pragma: no-cache
Content-Type: text/html


<select name="obj_id" onChange="showimage()">
<option value="">== Select ==</option>
$selector
</select>
END
}
sub getImage{
	#return &getObject("image",$data{content_id});
	return $dlg->get_object("image",$data{content_id});
}
sub getUrl{
	#return &getObject("url",$data{content_id});
	return $dlg->get_object("url",$data{content_id});
}
# create a library drop down list
sub getLibrarySelector{
	my ($out,@libs,$s);
	@libs = @{$dlg->fetch_library($data{section_id})};
	my $selector;
	foreach my $l (@libs){
		$selector .= "\n<option value=\"$l->{page_id}\"";
		# set first image library as the default
		if ($l->{filename} =~ /image/i && !$selectedlib){
			$selector .= " selected";
			$selectedlib = $l->{page_id};
		}
		$selector .= ">$l->{filename}</option>";
	}
	return <<END;
<b>Use library:</b><br>
<select id="lib_id" name="lib_id" onChange="getImageList()">
<option value="">== Select ==</option>
$selector
</select>
END
}
# create a default image/document drop down list
sub getImageSelector{
	my ($dlist) = @_;
	return <<END;
<b>Select an image/document:</b><br>
<div id="imageSelector">
<select name="obj_id" onChange="showimage()">
<option value="">== Select ==</option>
</select>
</div>
END
}
# create an image alignment drop down list
sub getAlignmentRadio{	
	return <<END;
<div id="img_alignment" style="visibility: hidden;">
<b>Image Alignment:</b><br>
<select name="align">
<option value="">== Select ==</option>
<option value="left">Left</option>
<option value="right">Right</option>
</select>
</div>
END
}
# preview area
sub getPreviewBlock{
	return <<END;
<b><center>Preview</center></b><br><div id="disp"></div>
END
}
# fetch image/document list for the default library
sub setDefaultLib{
	if ($selectedlib){
		return <<END;
<script language="javascript">
<!--
document.forms[0].lib_id.value = $selectedlib;
getImageList();
//-->
</script>
END
	}
	return;
}
### END
